#pragma once

#include <vector>
#include <iostream.h>
#include "vec3.h"

using namespace std;

/**
 * Class which represents one streamline. This class contains all points
 * and methods to check other points.
 * @author Christian Hirsch
 * @author Clemens Roegner
 */
class Streamline
{
public:
	Streamline();
	~Streamline(void);

	/**
	 * Adds a new point to the streamline.
	 * @param pos The new position.
	 */
	void addPointBack(vec3 pos);
	/**
	 * Checks a point, if it is close to this streamline.
	 * @param pos Position to check.
	 * @param dist The test distance.
	 * @param delta The offset of points which are not tested.
	 * @retun False if a point is too close to this streamline.
	 */
	bool checkPoint(vec3 pos, float dist, int delta);
	/**
	 * Iterates throw all points.
	 * @param *valid Pointer to a variable in which is set, if the last candidate has been reached.
	 * @return The next point.
	 */
	vec3 getNextCandidate(bool *valid);

	/**
	 * Gets the number of points which are stored in this streamline.
	 * @return Number of stored points.
	 */
	int getPointSize();

private:
	vector<vec3> points;

	int pointPos;
};
