#include <QGraphicsScene>
#include <QPainter>
#include <QColorDialog>

#include "TransferPoint.hpp"

TransferPoint::TransferPoint(qreal relX, qreal relY)
{
	setRelativePos(relX, relY);
	isFixed = false;
	if(relX == 0.0 || relX == 1.0)
		isFixed = true;
	setAcceptedMouseButtons(Qt::LeftButton);
}

QRectF TransferPoint::boundingRect() const
{
	return QRectF(-3, -3, 7, 7);
}

void TransferPoint::updateScene()
{
	setPos(relX * scene()->width(),
			relY * scene()->height());
}

void TransferPoint::paint (QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0)
{
	painter->setPen(QPen(Qt::black, 1));
	painter->setBrush(color);
	painter->drawRoundRect(-3,-3,7,7,2,2);
}

void TransferPoint::updatePosition(qreal _x, qreal _y)
{
	setPos(isFixed ? x() : _x,
			_y);

	relX = x()/scene()->width();
	relY = y()/scene()->height();
}

void TransferPoint::setFixed(bool fixed)
{
	isFixed = fixed;
}

void TransferPoint::setRelativePos(qreal x, qreal y)
{
	relX = x;
	relY = y;
}

qreal TransferPoint::getRelativeX()
{
	return relX;
}

qreal TransferPoint::getRelativeY()
{
	return relY;
}

void TransferPoint::setColor(QColor _color = Qt::white)
{
	color = _color;
}

void TransferPoint::setColorFromDialog()
{
	color = QColorDialog::getColor();
}

QColor TransferPoint::getColor()
{
	QColor output = color;
	output.setAlphaF(1-relY);
	return output;
}

void TransferPoint::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
}

void TransferPoint::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
	qreal _x = event->scenePos().x();
	qreal _y = event->scenePos().y();

	_x = _x < 1 ? 1 :
			_x > scene()->width()-1 ? scene()->width()-1 : _x;
	_y = _y < 0 ? 0 :
			_y > scene()->height() ? scene()->height() : _y;

	updatePosition(_x, _y);
}
