#include "vis_lab2.h"

Vis_Lab2::Vis_Lab2(QWidget *parent)
: QWidget(parent)
{
	createMenu();
	createGLGroupBox();
	createTransferGroupBox();
	createFileGroupBox();

	QHBoxLayout *mainLayout = new QHBoxLayout;
	QVBoxLayout *leftLayout = new QVBoxLayout;

	mainLayout->addLayout(leftLayout);
	mainLayout->addWidget(glGroupBox);
	mainLayout->setMenuBar(menuBar);

	leftLayout->addWidget(transferGroupBox);
	leftLayout->addWidget(fileGroupBox);

	setLayout(mainLayout);

	setWindowTitle(tr("Visualisierung - 2. Beispiel"));
}

Vis_Lab2::~Vis_Lab2()
{

}

void Vis_Lab2::createMenu()
{
	menuBar = new QMenuBar;

	fileMenu = new QMenu(tr("&Datei"));
	QAction *exitAction = fileMenu->addAction(tr("&Exit"));
	menuBar->addMenu(fileMenu);

	connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));
}

void Vis_Lab2::createGLGroupBox()
{
	glGroupBox = new QGroupBox(tr("OpenGL Rendering Fenster"));
	glWidget = new GLWidget;

	QHBoxLayout *layout = new QHBoxLayout;
	layout->addWidget(glWidget);
	connect(glWidget, SIGNAL(numChannelsChanged(int)),
		this, SLOT(numChannelsChanged(int)));
	glGroupBox->setLayout(layout);
}

void Vis_Lab2::createTransferGroupBox()
{
	transferGroupBox = new QGroupBox(tr("Transferfunktion"));
	//transferGroupBox->setMaximumHeight(200);
	QVBoxLayout *mainLayout = new QVBoxLayout;
	transferGroupBox->setLayout(mainLayout);

	tWidget = new TransferWidget(this);
	connect(tWidget, SIGNAL(transferChanged()), this, SLOT(transferChanged()));
	mainLayout->addWidget(tWidget);

	// channels
	QFormLayout *channelLayout = new QFormLayout;
	channelComboBox = new QComboBox;
	connect(channelComboBox, SIGNAL(currentIndexChanged(int)),
		glWidget, SLOT(channelChanged(int)));
	channelLayout->addRow(new QLabel(tr("Channel:")), channelComboBox);
	mainLayout->addLayout(channelLayout);

	tabWidget = new QTabWidget;
	mainLayout->addWidget(tabWidget);

	

	// arrows
	QWidget *arrowWidget = new QWidget;
	QFormLayout *arrowLayout = new QFormLayout;
	arrowWidget->setLayout(arrowLayout);
	tabWidget->addTab(arrowWidget, tr("Arrowplot"));

	arrowCheckBox = new QCheckBox;
	connect(arrowCheckBox, SIGNAL(toggled(bool)),
		glWidget, SLOT(switchArrowState(bool)));
	arrowLayout->addRow(new QLabel(tr("Arrowplot:")),
		arrowCheckBox);

	arrowNumSlider = new QSlider(Qt::Horizontal);
	arrowNumSlider->setMinimum(2);
	arrowNumSlider->setMaximum(50);
	arrowNumSlider->setValue(5);
	arrowNumSlider->setSingleStep(1);
	connect(arrowNumSlider, SIGNAL(valueChanged(int)),
		glWidget, SLOT(numArrowChanged(int)));
	arrowLayout->addRow(new QLabel(tr("Number of Arrows:")),
		arrowNumSlider);

	// arrow color
	arrowColorBtn = new QPushButton;
	arrowColorBtn->setFixedSize(20, 20);
	connect(arrowColorBtn, SIGNAL(clicked()),
		this, SLOT(arrowColorChanged()));
	arrowAlphaSlider = new QSlider(Qt::Horizontal);
	arrowAlphaSlider->setMinimum(0);
	arrowAlphaSlider->setMaximum(10);
	arrowAlphaSlider->setValue(10);
	arrowAlphaSlider->setSingleStep(1);
	connect(arrowAlphaSlider, SIGNAL(valueChanged(int)),
		glWidget, SLOT(alphaArrowChanged(int)));

	arrowLayout->addRow(new QLabel(tr("Color:")), arrowColorBtn);
	arrowLayout->addRow(new QLabel(tr("Alpha:")), arrowAlphaSlider);

	// streamlines
	QWidget *streamWidget = new QWidget;
	QFormLayout *streamLayout = new QFormLayout;
	streamWidget->setLayout(streamLayout);
	tabWidget->addTab(streamWidget, tr("Streamplot"));

	streamCheckBox = new QCheckBox;
	connect(streamCheckBox, SIGNAL(toggled(bool)),
		glWidget, SLOT(switchStreamlineState(bool)));
	streamLayout->addRow(new QLabel(tr("Streamlines:")),
		streamCheckBox);

	dtSpinBox = new QDoubleSpinBox;
	dtSpinBox->setDecimals(3);
	dtSpinBox->setSingleStep(0.001);
	dtSpinBox->setMinimum(0.001);
	dtSpinBox->setValue(1.0f);
	connect(dtSpinBox, SIGNAL(valueChanged(double)),
		glWidget, SLOT(dtValueChanged(double)));
	streamLayout->addRow(new QLabel(tr("dt:")),
		dtSpinBox);

	dsepSpinBox = new QDoubleSpinBox;
	dsepSpinBox->setDecimals(3);
	dsepSpinBox->setSingleStep(0.001);
	dsepSpinBox->setMinimum(0.001);
	dsepSpinBox->setValue(1.0f);
	connect(dsepSpinBox, SIGNAL(valueChanged(double)),
		glWidget, SLOT(dsepValueChanged(double)));
	streamLayout->addRow(new QLabel(tr("dsep:")),
		dsepSpinBox);

	dtestSpinBox = new QDoubleSpinBox;
	dtestSpinBox->setDecimals(3);
	dtestSpinBox->setSingleStep(0.001);
	dtestSpinBox->setMinimum(0.001);
	dtestSpinBox->setValue(0.5f);
	connect(dtestSpinBox, SIGNAL(valueChanged(double)),
		glWidget, SLOT(dtestValueChanged(double)));
	streamLayout->addRow(new QLabel(tr("dtest:")),
		dtestSpinBox);

	QGroupBox *interpolationGroup = new QGroupBox;
	QVBoxLayout *interpolationLayout = new QVBoxLayout;
	interpolationGroup->setLayout(interpolationLayout);
	eulerRadioBtn = new QRadioButton(tr("Euler"));
	eulerRadioBtn->setChecked(true);
	connect(eulerRadioBtn, SIGNAL(toggled(bool)),
		glWidget, SLOT(interpolationMethodChanged(bool)));
	interpolationLayout->addWidget(eulerRadioBtn);

	rungekuttaRadioBtn = new QRadioButton(tr("Runge-Kutta"));
	rungekuttaRadioBtn->setChecked(false);
	interpolationLayout->addWidget(rungekuttaRadioBtn);

	streamLayout->addRow(new QLabel(tr("Interpolation:")),
		interpolationGroup);

	QGroupBox *linestyleGroup = new QGroupBox;
	QVBoxLayout *linestyleLayout = new QVBoxLayout;
	linestyleGroup->setLayout(linestyleLayout);

	normalLineRadioBtn = new QRadioButton(tr("normal"));
	normalLineRadioBtn->setChecked(true);
	linestyleLayout->addWidget(normalLineRadioBtn);
	glyphMappingRadioBtn = new QRadioButton(tr("glyphmapping"));
	connect(glyphMappingRadioBtn, SIGNAL(toggled(bool)),
		glWidget, SLOT(paintMethodChanged(bool)));
	linestyleLayout->addWidget(glyphMappingRadioBtn);

	streamLayout->addRow(new QLabel(tr("Linestyle:")),
		linestyleGroup);


	// arrow color
	streamColorBtn = new QPushButton;
	streamColorBtn->setFixedSize(20, 20);
	connect(streamColorBtn, SIGNAL(clicked()),
		this, SLOT(streamColorChanged()));
	streamAlphaSlider = new QSlider(Qt::Horizontal);
	streamAlphaSlider->setMinimum(0);
	streamAlphaSlider->setMaximum(10);
	streamAlphaSlider->setValue(10);
	streamAlphaSlider->setSingleStep(1);
	connect(streamAlphaSlider, SIGNAL(valueChanged(int)),
		glWidget, SLOT(alphaStreamChanged(int)));

	streamLayout->addRow(new QLabel(tr("Color:")), streamColorBtn);
	streamLayout->addRow(new QLabel(tr("Alpha:")), streamAlphaSlider);
}

void Vis_Lab2::createFileGroupBox()
{
	fileGroupBox = new QGroupBox(tr("Dateiverwatlung"));
	QVBoxLayout *layout = new QVBoxLayout;
	fileGroupBox->setLayout(layout);

	txtFileLocation = new QLineEdit;
	btnOpenFileLocation = new QPushButton(tr("Strmungsdaten ffnen"));
	layout->addWidget(txtFileLocation);
	layout->addWidget(btnOpenFileLocation);

	connect(btnOpenFileLocation, SIGNAL(pressed()),
		this, SLOT(openFileLocation()));

	btnSaveScreenshot = new QPushButton(tr("Screenshot speichern"));
	connect(btnSaveScreenshot, SIGNAL(pressed()),
		glWidget, SLOT(saveScreenshot()));
	layout->addWidget(btnSaveScreenshot);
}

void Vis_Lab2::openFileLocation()
{
	txtFileLocation->setText(QFileDialog::getOpenFileName(this));
	if(txtFileLocation->text().isEmpty())
		return;

	glWidget->loadData(txtFileLocation->text().toStdString());
}

void Vis_Lab2::transferChanged()
{
	glWidget->setTransferValues(tWidget->getTransferValues());
}

void Vis_Lab2::numChannelsChanged(int _num)
{
	channelComboBox->clear();
	for(int i=0; i < _num; i++)
	{
		char buf[2];
		sprintf(buf, "%d", i);
		channelComboBox->addItem(buf);
	}
}

void Vis_Lab2::arrowColorChanged()
{
	QPixmap fillColor;
	QColor color(QColorDialog::getColor());
	fillColor.fill(color);
	arrowColorBtn->setMask(fillColor.mask());

	glWidget->setArrowColor((float)color.redF(), (float)color.greenF(), (float)color.blueF());
}

void Vis_Lab2::streamColorChanged()
{
	QPixmap fillColor;
	QColor color(QColorDialog::getColor());
	fillColor.fill(color);
	streamColorBtn->setMask(fillColor.mask());

	glWidget->setStreamColor((float)color.redF(), (float)color.greenF(), (float)color.blueF());
}