#ifndef VIS_LAB2_H
#define VIS_LAB2_H

#include <QtGui/QWidget>

#include "TransferWidget.hpp"
#include "GLWidget.hpp"

/**
 * The main GUI class whicht has all needed GUI elements.
 * @author Christian Hirsch
 * @author Clemens Roegner
 */
class Vis_Lab2 : public QWidget
{
	Q_OBJECT

private:

	/**
	 * Creates the menu.
	 * This function creates a new file-menu.
	 */
	void createMenu();
	/**
	 * Creates the goup box for the OpenGL window.
	 */
	void createGLGroupBox();
	/**
	 * Creates the group box, which contains the widget for the transfer function
	 * and several widgets for changing the arrow- and streamlineplot.
	 */
	void createTransferGroupBox();
	/**
	 * Creates a new group box for file handling. To load data and save screenshots.
	 */
	void createFileGroupBox();

	QMenu *fileMenu;
	QMenuBar *menuBar;

	// render window
	QGroupBox *glGroupBox;
	GLWidget *glWidget;

	// transfer function
	QGroupBox *transferGroupBox;
	QComboBox *channelComboBox;
	TransferWidget *tWidget;

	QTabWidget *tabWidget;

	QCheckBox *streamCheckBox;
	QDoubleSpinBox *dtSpinBox;
	QDoubleSpinBox *dsepSpinBox;
	QDoubleSpinBox *dtestSpinBox;
	QRadioButton *eulerRadioBtn;
	QRadioButton *rungekuttaRadioBtn;
	QRadioButton *normalLineRadioBtn;
	QRadioButton *glyphMappingRadioBtn;
	QPushButton *streamColorBtn;
	QSlider *streamAlphaSlider;

	QCheckBox *arrowCheckBox;
	QSlider *arrowNumSlider;
	QPushButton *arrowColorBtn;
	QSlider *arrowAlphaSlider;

	// file handling
	QGroupBox *fileGroupBox;
	QLineEdit *txtFileLocation;
	QPushButton *btnOpenFileLocation;
	QPushButton *btnSaveScreenshot;

public:
	Vis_Lab2(QWidget *parent = 0);
	~Vis_Lab2();

public slots:
	/**
	 * Opens new flow data and processes it to the opengl rendering window.
	 */
	void openFileLocation();
	/**
	 * Processes the new transferfunction values to the openglrendering window.
	 */
	void transferChanged();
	/**
	 * Updates the number of channels to select all available channels.
	 */
	void numChannelsChanged(int);
	/**
	 * Changes the color of the arrows.
	 */
	void arrowColorChanged();
	/**
	 * Changes the color of the arrows.
	 */
	void streamColorChanged();
};

#endif // VIS_LAB2_H
